/**
 * Copyright © 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.deductiondetails.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.deductiondetails.entity.EvaluationSummary;
import com.jeeplus.modules.deductiondetails.entity.SelectClass;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.deductiondetails.entity.DeductionDetails;
import com.jeeplus.modules.deductiondetails.service.DeductionDetailsService;

/**
 * deductionDetailsController
 * @author 邱悦
 * @version 2022-01-01
 */
@RestController
@RequestMapping(value = "/deductiondetails/deductionDetails")
public class DeductionDetailsController extends BaseController {

	@Autowired
	private DeductionDetailsService deductionDetailsService;

	@ModelAttribute
	public DeductionDetails get(@RequestParam(required=false) String id) {
		DeductionDetails entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = deductionDetailsService.get(id);
		}
		if (entity == null){
			entity = new DeductionDetails();
		}
		return entity;
	}

	/**
	 * 扣分详情列表数据
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("list")
	public AjaxJson list(DeductionDetails deductionDetails, HttpServletRequest request, HttpServletResponse response) {
		Page<DeductionDetails> page = deductionDetailsService.findPage(new Page<DeductionDetails>(request, response), deductionDetails);
		return AjaxJson.success().put("page",page);
	}

	/**
	 * 施工单位评价-1汇总情况
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("summaryList")
	public AjaxJson summaryList(EvaluationSummary evaluationSummary, HttpServletRequest request, HttpServletResponse response) {
		Page<EvaluationSummary> page = deductionDetailsService.findSummeryPage(new Page<EvaluationSummary>(request, response), evaluationSummary);
		return AjaxJson.success().put("page",page);
	}


	/**
	 * 施工单位评价-1汇总情况
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("getSummary")
	public AjaxJson getSummary(EvaluationSummary evaluationSummary, HttpServletRequest request, HttpServletResponse response) {
		EvaluationSummary deductionDetails = deductionDetailsService.getSummary(evaluationSummary);
		return AjaxJson.success().put("deductionDetails",deductionDetails);
	}

	/**
	 * 大类选取
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("getBigClass")
	public AjaxJson getBigClass(String id,HttpServletRequest request, HttpServletResponse response) {
		List<SelectClass> bigClass = deductionDetailsService.getBigClass(id);
		return AjaxJson.success().put("bigClass",bigClass);
	}

	/**
	 * 小类选取
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("getSubClass")
	public AjaxJson getSubClass(String id,HttpServletRequest request, HttpServletResponse response) {
		List<SelectClass> subClass = deductionDetailsService.getSubClass(id);
		return AjaxJson.success().put("subClass",subClass);
	}

	/**
	 * 获取当前小类是否可以否决
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:list")
	@GetMapping("getVeto")
	public AjaxJson getVeto(String id,HttpServletRequest request, HttpServletResponse response) {
		String veto = deductionDetailsService.getVeto(id);
		return AjaxJson.success().put("veto",veto);
	}

	/**
	 * 根据Id获取扣分详情数据
	 */
	@RequiresPermissions(value={"deductiondetails:deductionDetails:view","deductiondetails:deductionDetails:add","deductiondetails:deductionDetails:edit"},logical=Logical.OR)
	@GetMapping("queryById")
	public AjaxJson queryById(DeductionDetails deductionDetails) {
		return AjaxJson.success().put("deductionDetails", deductionDetails);
	}

	/**
	 * 保存扣分详情
	 */
	@RequiresPermissions(value={"deductiondetails:deductionDetails:add","deductiondetails:deductionDetails:edit"},logical=Logical.OR)
	@PostMapping("save")
	public AjaxJson save(DeductionDetails deductionDetails, Model model) throws Exception{
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(deductionDetails);
		if (StringUtils.isNotBlank(errMsg)){
			return AjaxJson.error(errMsg);
		}
		//新增或编辑表单保存
		deductionDetailsService.save(deductionDetails);//保存
		return AjaxJson.success("保存扣分详情成功");
	}

	/**
	 * 保存扣分详情
	 */
	@RequiresPermissions(value={"deductiondetails:deductionDetails:add","deductiondetails:deductionDetails:edit"},logical=Logical.OR)
	@PostMapping("insertDetails")
	public AjaxJson insertDetails(DeductionDetails deductionDetails, Model model) throws Exception{
		deductionDetailsService.insertDetails(deductionDetails);
		return AjaxJson.success("保存扣分详情成功");
	}


	/**
	 * 批量删除扣分详情
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:del")
	@DeleteMapping("delete")
	public AjaxJson delete(String ids) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			deductionDetailsService.delete(deductionDetailsService.get(id));
		}
		return AjaxJson.success("删除扣分详情成功");
	}

	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:export")
    @GetMapping("export")
    public AjaxJson exportFile(EvaluationSummary evaluationSummary, HttpServletRequest request, HttpServletResponse response) {
		try {
            String fileName = "评分汇总"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<EvaluationSummary> page = deductionDetailsService.findSummeryPage(new Page<EvaluationSummary>(request, response, -1), evaluationSummary);
    		new ExportExcel("评分汇总", EvaluationSummary.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			return AjaxJson.error("导出评分汇总记录失败！失败信息："+e.getMessage());
		}
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("deductiondetails:deductionDetails:import")
    @PostMapping("import")
   	public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<DeductionDetails> list = ei.getDataList(DeductionDetails.class);
			for (DeductionDetails deductionDetails : list){
				try{
					deductionDetailsService.save(deductionDetails);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条扣分详情记录。");
			}
			return AjaxJson.success( "已成功导入 "+successNum+" 条扣分详情记录"+failureMsg);
		} catch (Exception e) {
			return AjaxJson.error("导入扣分详情失败！失败信息："+e.getMessage());
		}
    }

	/**
	 * 下载导入扣分详情数据模板
	 */
	@RequiresPermissions("deductiondetails:deductionDetails:import")
    @GetMapping("import/template")
     public AjaxJson importFileTemplate(HttpServletResponse response) {
		try {
            String fileName = "扣分详情数据导入模板.xlsx";
    		List<DeductionDetails> list = Lists.newArrayList();
    		new ExportExcel("扣分详情数据", DeductionDetails.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			return AjaxJson.error( "导入模板下载失败！失败信息："+e.getMessage());
		}
    }


}