/**
 * Copyright © 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.deductionshowdetails.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.deductionshowdetails.entity.*;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.deductionshowdetails.service.DeductionShowDetailsService;

/**
 * DeductionShowDetailsController
 * @author 邱悦
 * @version 2022-01-02
 */
@RestController
@RequestMapping(value = "/deductionShowDetails/deductionShowDetails")
public class DeductionShowDetailsController extends BaseController {

	@Autowired
	private DeductionShowDetailsService deductionShowDetailsService;

	@ModelAttribute
	public DeductionShowDetails get(@RequestParam(required=false) String id) {
		DeductionShowDetails entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = deductionShowDetailsService.get(id);
		}
		if (entity == null){
			entity = new DeductionShowDetails();
		}
		return entity;
	}

	/**
	 * 扣分详细列表数据
	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:list")
	@GetMapping("list")
	public AjaxJson list(DeductionShowDetails deductionShowDetails, HttpServletRequest request, HttpServletResponse response) {
		Page<DeductionShowDetails> page = deductionShowDetailsService.findPage(new Page<DeductionShowDetails>(request, response), deductionShowDetails);
		return AjaxJson.success().put("page",page);
	}

	/**
	 * 扣分详细列表数据
	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:list")
	@GetMapping("getName")
	public AjaxJson getName(String  id, HttpServletRequest request, HttpServletResponse response) {
		String name = deductionShowDetailsService.getName(id);
		return AjaxJson.success().put("unitName", name);
	}

	/**
	 * 根据Id获取扣分详细数据
	 */
	@RequiresPermissions(value={"deductionShowDetails:deductionShowDetails:view","deductionShowDetails:deductionShowDetails:add","deductionShowDetails:deductionShowDetails:edit"},logical=Logical.OR)
	@GetMapping("queryById")
	public AjaxJson queryById(DeductionShowDetails deductionShowDetails) {
		return AjaxJson.success().put("deductionShowDetails", deductionShowDetails);
	}

	/**
	 * 保存扣分详细
	 */
	@RequiresPermissions(value={"deductionShowDetails:deductionShowDetails:add","deductionShowDetails:deductionShowDetails:edit"},logical=Logical.OR)
	@PostMapping("save")
	public AjaxJson save(DeductionShowDetails deductionShowDetails, Model model) throws Exception{
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(deductionShowDetails);
		if (StringUtils.isNotBlank(errMsg)){
			return AjaxJson.error(errMsg);
		}
		//新增或编辑表单保存
		deductionShowDetailsService.save(deductionShowDetails);//保存
		return AjaxJson.success("保存扣分详细成功");
	}

	/**
	 * 保存扣分详细
	 */
	@RequiresPermissions(value={"deductionShowDetails:deductionShowDetails:add","deductionShowDetails:deductionShowDetails:edit"},logical=Logical.OR)
	@PostMapping("update")
	public AjaxJson update(DeductionShowDetails deductionShowDetails, Model model) throws Exception{
		//新增或编辑表单保存
		deductionShowDetailsService.update(deductionShowDetails);//保存
		return AjaxJson.success("保存扣分详细成功");
	}

	/**
	 * 批量删除扣分详细
	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:del")
	@DeleteMapping("delete")
	public AjaxJson delete(String ids) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			deductionShowDetailsService.deleteDetails(id);
		}
		return AjaxJson.success("删除扣分详细成功");
	}

	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:export")
    @GetMapping("export")
    public AjaxJson exportFile(DeductionShowDetails deductionShowDetails, HttpServletRequest request, HttpServletResponse response) {
		try {
            String fileName = "扣分详细"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<DeductionShowDetails> page = deductionShowDetailsService.findPage(new Page<DeductionShowDetails>(request, response, -1), deductionShowDetails);
    		new ExportExcel("扣分详细", DeductionShowDetails.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			return AjaxJson.error("导出扣分详细记录失败！失败信息："+e.getMessage());
		}
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:import")
    @PostMapping("import")
   	public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<DeductionShowDetails> list = ei.getDataList(DeductionShowDetails.class);
			for (DeductionShowDetails deductionShowDetails : list){
				try{
					deductionShowDetailsService.save(deductionShowDetails);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条扣分详细记录。");
			}
			return AjaxJson.success( "已成功导入 "+successNum+" 条扣分详细记录"+failureMsg);
		} catch (Exception e) {
			return AjaxJson.error("导入扣分详细失败！失败信息："+e.getMessage());
		}
    }

	/**
	 * 下载导入扣分详细数据模板
	 */
	@RequiresPermissions("deductionShowDetails:deductionShowDetails:import")
    @GetMapping("import/template")
     public AjaxJson importFileTemplate(HttpServletResponse response) {
		try {
            String fileName = "扣分详细数据导入模板.xlsx";
    		List<DeductionShowDetails> list = Lists.newArrayList();
    		new ExportExcel("扣分详细数据", DeductionShowDetails.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			return AjaxJson.error( "导入模板下载失败！失败信息："+e.getMessage());
		}
    }




	/**
	 * 扣分详细列表数据
	 */
	@GetMapping("siteQualityList")
	public AjaxJson siteQualityList(SiteQuality siteQuality, HttpServletRequest request, HttpServletResponse response) {
		Page<SiteQuality> page = deductionShowDetailsService.findSiteQualityList(new Page<SiteQuality>(request, response), siteQuality);
		return AjaxJson.success().put("page",page);
	}

	/**
	 * 扣分详细列表数据
	 */
	@GetMapping("siteSafetyList")
	public AjaxJson siteSafetyList(SiteSafety siteSafety, HttpServletRequest request, HttpServletResponse response) {
		Page<SiteSafety> page = deductionShowDetailsService.findSiteSafetyList(new Page<SiteSafety>(request, response), siteSafety);
		return AjaxJson.success().put("page",page);
	}

	/**
	 * 扣分详细列表数据
	 */
	@GetMapping("costCompletionList")
	public AjaxJson costCompletionList(CostCompletion costCompletion, HttpServletRequest request, HttpServletResponse response) {
		Page<CostCompletion> page = deductionShowDetailsService.findCostCompletionList(new Page<CostCompletion>(request, response), costCompletion);
		return AjaxJson.success().put("page",page);
	}
	/**
	 * 扣分详细列表数据
	 */
	@GetMapping("qualityCivilizedList")
	public AjaxJson qualityCivilizedList(QualityCivilized qualityCivilized, HttpServletRequest request, HttpServletResponse response) {
		Page<QualityCivilized> page = deductionShowDetailsService.findQualityCivilizedList(new Page<QualityCivilized>(request, response), qualityCivilized);
		return AjaxJson.success().put("page",page);
	}
	/**
	 * 扣分详细列表数据
	 */
	@GetMapping("otherPointsList")
	public AjaxJson otherPointsList(OtherPoints otherPoints, HttpServletRequest request, HttpServletResponse response) {
		Page<OtherPoints> page = deductionShowDetailsService.findOtherPointsList(new Page<OtherPoints>(request, response), otherPoints);
		return AjaxJson.success().put("page",page);
	}

}